@@ -12,9 +12,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -27,7 +25,7 @@
 #ifndef __GTK_TEXT_VIEW_H__
 #define __GTK_TEXT_VIEW_H__
 
-#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
 #error "Only <gtk/gtk.h> can be included directly."
 #endif
 
@@ -45,9 +43,22 @@ G_BEGIN_DECLS
 #define GTK_IS_TEXT_VIEW_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_TEXT_VIEW))
 #define GTK_TEXT_VIEW_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_TEXT_VIEW, GtkTextViewClass))
 
+/**
+ * GtkTextWindowType:
+ * @GTK_TEXT_WINDOW_WIDGET: Window that floats over scrolling areas.
+ * @GTK_TEXT_WINDOW_TEXT: Scrollable text window.
+ * @GTK_TEXT_WINDOW_LEFT: Left side border window.
+ * @GTK_TEXT_WINDOW_RIGHT: Right side border window.
+ * @GTK_TEXT_WINDOW_TOP: Top border window.
+ * @GTK_TEXT_WINDOW_BOTTOM: Bottom border window.
+ *
+ * Used to reference the parts of #GtkTextView.
+ */
 typedef enum
 {
+  /*< private >*/
   GTK_TEXT_WINDOW_PRIVATE,
+  /*< public >*/
   GTK_TEXT_WINDOW_WIDGET,
   GTK_TEXT_WINDOW_TEXT,
   GTK_TEXT_WINDOW_LEFT,
@@ -56,112 +67,31 @@ typedef enum
   GTK_TEXT_WINDOW_BOTTOM
 } GtkTextWindowType;
 
+/**
+ * GTK_TEXT_VIEW_PRIORITY_VALIDATE:
+ *
+ * The priority at which the text view validates onscreen lines
+ * in an idle job in the background.
+ */
 #define GTK_TEXT_VIEW_PRIORITY_VALIDATE (GDK_PRIORITY_REDRAW + 5)
 
-typedef struct _GtkTextView GtkTextView;
-typedef struct _GtkTextViewClass GtkTextViewClass;
-
-/* Internal private types. */
-typedef struct _GtkTextWindow GtkTextWindow;
-typedef struct _GtkTextPendingScroll GtkTextPendingScroll;
+typedef struct _GtkTextView        GtkTextView;
+typedef struct _GtkTextViewPrivate GtkTextViewPrivate;
+typedef struct _GtkTextViewClass   GtkTextViewClass;
 
 struct _GtkTextView
 {
   GtkContainer parent_instance;
 
-  struct _GtkTextLayout *GSEAL (layout);
-  GtkTextBuffer *GSEAL (buffer);
-
-  guint GSEAL (selection_drag_handler);
-  guint GSEAL (scroll_timeout);
-
-  /* Default style settings */
-  gint GSEAL (pixels_above_lines);
-  gint GSEAL (pixels_below_lines);
-  gint GSEAL (pixels_inside_wrap);
-  GtkWrapMode GSEAL (wrap_mode);
-  GtkJustification GSEAL (justify);
-  gint GSEAL (left_margin);
-  gint GSEAL (right_margin);
-  gint GSEAL (indent);
-  PangoTabArray *GSEAL (tabs);
-  guint GSEAL (editable) : 1;
-
-  guint GSEAL (overwrite_mode) : 1;
-  guint GSEAL (cursor_visible) : 1;
-
-  /* if we have reset the IM since the last character entered */  
-  guint GSEAL (need_im_reset) : 1;
-
-  guint GSEAL (accepts_tab) : 1;
-
-  guint GSEAL (width_changed) : 1;
-
-  /* debug flag - means that we've validated onscreen since the
-   * last "invalidate" signal from the layout
-   */
-  guint GSEAL (onscreen_validated) : 1;
-
-  guint GSEAL (mouse_cursor_obscured) : 1;
-
-  GtkTextWindow *GSEAL (text_window);
-  GtkTextWindow *GSEAL (left_window);
-  GtkTextWindow *GSEAL (right_window);
-  GtkTextWindow *GSEAL (top_window);
-  GtkTextWindow *GSEAL (bottom_window);
-
-  GtkAdjustment *GSEAL (hadjustment);
-  GtkAdjustment *GSEAL (vadjustment);
-
-  gint GSEAL (xoffset);         /* Offsets between widget coordinates and buffer coordinates */
-  gint GSEAL (yoffset);
-  gint GSEAL (width);           /* Width and height of the buffer */
-  gint GSEAL (height);
-
-  /* The virtual cursor position is normally the same as the
-   * actual (strong) cursor position, except in two circumstances:
-   *
-   * a) When the cursor is moved vertically with the keyboard
-   * b) When the text view is scrolled with the keyboard
-   *
-   * In case a), virtual_cursor_x is preserved, but not virtual_cursor_y
-   * In case b), both virtual_cursor_x and virtual_cursor_y are preserved.
-   */
-  gint GSEAL (virtual_cursor_x);   /* -1 means use actual cursor position */
-  gint GSEAL (virtual_cursor_y);   /* -1 means use actual cursor position */
-
-  GtkTextMark *GSEAL (first_para_mark); /* Mark at the beginning of the first onscreen paragraph */
-  gint GSEAL (first_para_pixels);       /* Offset of top of screen in the first onscreen paragraph */
-
-  GtkTextMark *GSEAL (dnd_mark);
-  guint GSEAL (blink_timeout);
-
-  guint GSEAL (first_validate_idle);        /* Idle to revalidate onscreen portion, runs before resize */
-  guint GSEAL (incremental_validate_idle);  /* Idle to revalidate offscreen portions, runs after redraw */
-
-  GtkIMContext *GSEAL (im_context);
-  GtkWidget *GSEAL (popup_menu);
-
-  gint GSEAL (drag_start_x);
-  gint GSEAL (drag_start_y);
-
-  GSList *GSEAL (children);
-
-  GtkTextPendingScroll *GSEAL (pending_scroll);
-
-  gint GSEAL (pending_place_cursor_button);
+  GtkTextViewPrivate *priv;
 };
 
 struct _GtkTextViewClass
 {
   GtkContainerClass parent_class;
 
-  void (* set_scroll_adjustments)   (GtkTextView    *text_view,
-                                     GtkAdjustment  *hadjustment,
-                                     GtkAdjustment  *vadjustment);
-
   void (* populate_popup)           (GtkTextView    *text_view,
-                                     GtkMenu        *menu);
+                                     GtkWidget      *popup);
 
   /* These are all RUN_ACTION signals for keybindings */
 
@@ -171,14 +101,6 @@ struct _GtkTextViewClass
                         gint            count,
                         gboolean        extend_selection);
 
-  /* FIXME should be deprecated in favor of adding GTK_MOVEMENT_HORIZONTAL_PAGES
-   * or something in GTK 2.2, was put in to avoid adding enum values during
-   * the freeze.
-   */
-  void (* page_horizontally) (GtkTextView *text_view,
-                              gint         count,
-                              gboolean     extend_selection);
-
   /* move the "anchor" (what Emacs calls the mark) to the cursor position */
   void (* set_anchor)  (GtkTextView    *text_view);
 
@@ -197,12 +119,7 @@ struct _GtkTextViewClass
   /* overwrite */
   void (* toggle_overwrite) (GtkTextView *text_view);
 
-  /* as of GTK+ 2.12 the "move-focus" signal has been moved to GtkWidget,
-   * so this is merley a virtual function now. Overriding it in subclasses
-   * continues to work though.
-   */
-  void (* move_focus)       (GtkTextView     *text_view,
-                             GtkDirectionType direction);
+  GtkTextBuffer * (* create_buffer) (GtkTextView *text_view);
 
   /* Padding for future expansion */
   void (*_gtk_reserved1) (void);
@@ -214,64 +131,89 @@ struct _GtkTextViewClass
   void (*_gtk_reserved7) (void);
 };
 
+GDK_AVAILABLE_IN_ALL
 GType          gtk_text_view_get_type              (void) G_GNUC_CONST;
+GDK_AVAILABLE_IN_ALL
 GtkWidget *    gtk_text_view_new                   (void);
+GDK_AVAILABLE_IN_ALL
 GtkWidget *    gtk_text_view_new_with_buffer       (GtkTextBuffer *buffer);
+GDK_AVAILABLE_IN_ALL
 void           gtk_text_view_set_buffer            (GtkTextView   *text_view,
                                                     GtkTextBuffer *buffer);
+GDK_AVAILABLE_IN_ALL
 GtkTextBuffer *gtk_text_view_get_buffer            (GtkTextView   *text_view);
+GDK_AVAILABLE_IN_ALL
 gboolean       gtk_text_view_scroll_to_iter        (GtkTextView   *text_view,
                                                     GtkTextIter   *iter,
                                                     gdouble        within_margin,
                                                     gboolean       use_align,
                                                     gdouble        xalign,
                                                     gdouble        yalign);
+GDK_AVAILABLE_IN_ALL
 void           gtk_text_view_scroll_to_mark        (GtkTextView   *text_view,
                                                     GtkTextMark   *mark,
                                                     gdouble        within_margin,
                                                     gboolean       use_align,
                                                     gdouble        xalign,
                                                     gdouble        yalign);
+GDK_AVAILABLE_IN_ALL
 void           gtk_text_view_scroll_mark_onscreen  (GtkTextView   *text_view,
                                                     GtkTextMark   *mark);
+GDK_AVAILABLE_IN_ALL
 gboolean       gtk_text_view_move_mark_onscreen    (GtkTextView   *text_view,
                                                     GtkTextMark   *mark);
+GDK_AVAILABLE_IN_ALL
 gboolean       gtk_text_view_place_cursor_onscreen (GtkTextView   *text_view);
 
+GDK_AVAILABLE_IN_ALL
 void           gtk_text_view_get_visible_rect      (GtkTextView   *text_view,
                                                     GdkRectangle  *visible_rect);
+GDK_AVAILABLE_IN_ALL
 void           gtk_text_view_set_cursor_visible    (GtkTextView   *text_view,
                                                     gboolean       setting);
+GDK_AVAILABLE_IN_ALL
 gboolean       gtk_text_view_get_cursor_visible    (GtkTextView   *text_view);
 
+GDK_AVAILABLE_IN_ALL
+void           gtk_text_view_get_cursor_locations  (GtkTextView       *text_view,
+                                                    const GtkTextIter *iter,
+                                                    GdkRectangle      *strong,
+                                                    GdkRectangle      *weak);
+GDK_AVAILABLE_IN_ALL
 void           gtk_text_view_get_iter_location     (GtkTextView   *text_view,
                                                     const GtkTextIter *iter,
                                                     GdkRectangle  *location);
+GDK_AVAILABLE_IN_ALL
 void           gtk_text_view_get_iter_at_location  (GtkTextView   *text_view,
                                                     GtkTextIter   *iter,
                                                     gint           x,
                                                     gint           y);
+GDK_AVAILABLE_IN_ALL
 void           gtk_text_view_get_iter_at_position  (GtkTextView   *text_view,
                                                     GtkTextIter   *iter,
 						    gint          *trailing,
                                                     gint           x,
                                                     gint           y);
+GDK_AVAILABLE_IN_ALL
 void           gtk_text_view_get_line_yrange       (GtkTextView       *text_view,
                                                     const GtkTextIter *iter,
                                                     gint              *y,
                                                     gint              *height);
 
+GDK_AVAILABLE_IN_ALL
 void           gtk_text_view_get_line_at_y         (GtkTextView       *text_view,
                                                     GtkTextIter       *target_iter,
                                                     gint               y,
                                                     gint              *line_top);
 
+GDK_AVAILABLE_IN_ALL
 void gtk_text_view_buffer_to_window_coords (GtkTextView       *text_view,
                                             GtkTextWindowType  win,
                                             gint               buffer_x,
                                             gint               buffer_y,
                                             gint              *window_x,
                                             gint              *window_y);
+GDK_AVAILABLE_IN_ALL
 void gtk_text_view_window_to_buffer_coords (GtkTextView       *text_view,
                                             GtkTextWindowType  win,
                                             gint               window_x,
@@ -279,43 +221,59 @@ void gtk_text_view_window_to_buffer_coords (GtkTextView       *text_view,
                                             gint              *buffer_x,
                                             gint              *buffer_y);
 
-GtkAdjustment* gtk_text_view_get_hadjustment (GtkTextView *text_view);
-GtkAdjustment* gtk_text_view_get_vadjustment (GtkTextView *text_view);
+GDK_DEPRECATED_IN_3_0_FOR(gtk_scrollable_get_hadjustment)
+GtkAdjustment*   gtk_text_view_get_hadjustment (GtkTextView   *text_view);
+GDK_DEPRECATED_IN_3_0_FOR(gtk_scrollable_get_vadjustment)
+GtkAdjustment*   gtk_text_view_get_vadjustment (GtkTextView   *text_view);
 
+GDK_AVAILABLE_IN_ALL
 GdkWindow*        gtk_text_view_get_window      (GtkTextView       *text_view,
                                                  GtkTextWindowType  win);
+GDK_AVAILABLE_IN_ALL
 GtkTextWindowType gtk_text_view_get_window_type (GtkTextView       *text_view,
                                                  GdkWindow         *window);
 
+GDK_AVAILABLE_IN_ALL
 void gtk_text_view_set_border_window_size (GtkTextView       *text_view,
                                            GtkTextWindowType  type,
                                            gint               size);
+GDK_AVAILABLE_IN_ALL
 gint gtk_text_view_get_border_window_size (GtkTextView       *text_view,
 					   GtkTextWindowType  type);
 
+GDK_AVAILABLE_IN_ALL
 gboolean gtk_text_view_forward_display_line           (GtkTextView       *text_view,
                                                        GtkTextIter       *iter);
+GDK_AVAILABLE_IN_ALL
 gboolean gtk_text_view_backward_display_line          (GtkTextView       *text_view,
                                                        GtkTextIter       *iter);
+GDK_AVAILABLE_IN_ALL
 gboolean gtk_text_view_forward_display_line_end       (GtkTextView       *text_view,
                                                        GtkTextIter       *iter);
+GDK_AVAILABLE_IN_ALL
 gboolean gtk_text_view_backward_display_line_start    (GtkTextView       *text_view,
                                                        GtkTextIter       *iter);
+GDK_AVAILABLE_IN_ALL
 gboolean gtk_text_view_starts_display_line            (GtkTextView       *text_view,
                                                        const GtkTextIter *iter);
+GDK_AVAILABLE_IN_ALL
 gboolean gtk_text_view_move_visually                  (GtkTextView       *text_view,
                                                        GtkTextIter       *iter,
                                                        gint               count);
 
+GDK_AVAILABLE_IN_ALL
 gboolean        gtk_text_view_im_context_filter_keypress        (GtkTextView       *text_view,
                                                                  GdkEventKey       *event);
+GDK_AVAILABLE_IN_ALL
 void            gtk_text_view_reset_im_context                  (GtkTextView       *text_view);
 
 /* Adding child widgets */
+GDK_AVAILABLE_IN_ALL
 void gtk_text_view_add_child_at_anchor (GtkTextView          *text_view,
                                         GtkWidget            *child,
                                         GtkTextChildAnchor   *anchor);
 
+GDK_AVAILABLE_IN_ALL
 void gtk_text_view_add_child_in_window (GtkTextView          *text_view,
                                         GtkWidget            *child,
                                         GtkTextWindowType     which_window,
@@ -323,6 +281,7 @@ void gtk_text_view_add_child_in_window (GtkTextView          *text_view,
                                         gint                  xpos,
                                         gint                  ypos);
 
+GDK_AVAILABLE_IN_ALL
 void gtk_text_view_move_child          (GtkTextView          *text_view,
                                         GtkWidget            *child,
                                         /* window coordinates */
@@ -331,46 +290,84 @@ void gtk_text_view_move_child          (GtkTextView          *text_view,
 
 /* Default style settings (fallbacks if no tag affects the property) */
 
+GDK_AVAILABLE_IN_ALL
 void             gtk_text_view_set_wrap_mode          (GtkTextView      *text_view,
                                                        GtkWrapMode       wrap_mode);
+GDK_AVAILABLE_IN_ALL
 GtkWrapMode      gtk_text_view_get_wrap_mode          (GtkTextView      *text_view);
+GDK_AVAILABLE_IN_ALL
 void             gtk_text_view_set_editable           (GtkTextView      *text_view,
                                                        gboolean          setting);
+GDK_AVAILABLE_IN_ALL
 gboolean         gtk_text_view_get_editable           (GtkTextView      *text_view);
+GDK_AVAILABLE_IN_ALL
 void             gtk_text_view_set_overwrite          (GtkTextView      *text_view,
 						       gboolean          overwrite);
+GDK_AVAILABLE_IN_ALL
 gboolean         gtk_text_view_get_overwrite          (GtkTextView      *text_view);
+GDK_AVAILABLE_IN_ALL
 void		 gtk_text_view_set_accepts_tab        (GtkTextView	*text_view,
 						       gboolean		 accepts_tab);
+GDK_AVAILABLE_IN_ALL
 gboolean	 gtk_text_view_get_accepts_tab        (GtkTextView	*text_view);
+GDK_AVAILABLE_IN_ALL
 void             gtk_text_view_set_pixels_above_lines (GtkTextView      *text_view,
                                                        gint              pixels_above_lines);
+GDK_AVAILABLE_IN_ALL
 gint             gtk_text_view_get_pixels_above_lines (GtkTextView      *text_view);
+GDK_AVAILABLE_IN_ALL
 void             gtk_text_view_set_pixels_below_lines (GtkTextView      *text_view,
                                                        gint              pixels_below_lines);
+GDK_AVAILABLE_IN_ALL
 gint             gtk_text_view_get_pixels_below_lines (GtkTextView      *text_view);
+GDK_AVAILABLE_IN_ALL
 void             gtk_text_view_set_pixels_inside_wrap (GtkTextView      *text_view,
                                                        gint              pixels_inside_wrap);
+GDK_AVAILABLE_IN_ALL
 gint             gtk_text_view_get_pixels_inside_wrap (GtkTextView      *text_view);
+GDK_AVAILABLE_IN_ALL
 void             gtk_text_view_set_justification      (GtkTextView      *text_view,
                                                        GtkJustification  justification);
+GDK_AVAILABLE_IN_ALL
 GtkJustification gtk_text_view_get_justification      (GtkTextView      *text_view);
+GDK_AVAILABLE_IN_ALL
 void             gtk_text_view_set_left_margin        (GtkTextView      *text_view,
                                                        gint              left_margin);
+GDK_AVAILABLE_IN_ALL
 gint             gtk_text_view_get_left_margin        (GtkTextView      *text_view);
+GDK_AVAILABLE_IN_ALL
 void             gtk_text_view_set_right_margin       (GtkTextView      *text_view,
                                                        gint              right_margin);
+GDK_AVAILABLE_IN_ALL
 gint             gtk_text_view_get_right_margin       (GtkTextView      *text_view);
+GDK_AVAILABLE_IN_ALL
 void             gtk_text_view_set_indent             (GtkTextView      *text_view,
                                                        gint              indent);
+GDK_AVAILABLE_IN_ALL
 gint             gtk_text_view_get_indent             (GtkTextView      *text_view);
+GDK_AVAILABLE_IN_ALL
 void             gtk_text_view_set_tabs               (GtkTextView      *text_view,
                                                        PangoTabArray    *tabs);
+GDK_AVAILABLE_IN_ALL
 PangoTabArray*   gtk_text_view_get_tabs               (GtkTextView      *text_view);
 
 /* note that the return value of this changes with the theme */
+GDK_AVAILABLE_IN_ALL
 GtkTextAttributes* gtk_text_view_get_default_attributes (GtkTextView    *text_view);
 
+GDK_AVAILABLE_IN_3_6
+void             gtk_text_view_set_input_purpose      (GtkTextView      *text_view,
+                                                       GtkInputPurpose   purpose);
+GDK_AVAILABLE_IN_3_6
+GtkInputPurpose  gtk_text_view_get_input_purpose      (GtkTextView      *text_view);
+
+GDK_AVAILABLE_IN_3_6
+void             gtk_text_view_set_input_hints        (GtkTextView      *text_view,
+                                                       GtkInputHints     hints);
+GDK_AVAILABLE_IN_3_6
+GtkInputHints    gtk_text_view_get_input_hints        (GtkTextView      *text_view);
+
+
 G_END_DECLS
 
 #endif /* __GTK_TEXT_VIEW_H__ */
